(function (Drupal, drupalSettings) {
  "use strict";
  Drupal.behaviors.product3d = {
    attach: function (context, settings) {
      console.log("[viewer_3d]", "Load Module");

      if (drupalSettings.product3d === undefined) return;

      // Acceder a las variables del bloque.
      //var title = drupalSettings.product3d.title;
      //var description = drupalSettings.product3d.description;
      // Acceder a las URLs de los archivos subidos.
      var modelFiles = drupalSettings.product3d.model_files;

      // Mostrar las variables en la consola (opcional).
      //console.log("Title:", title);
      //console.log("Description:", description);
      console.log("Model File URL:", modelFiles);

      document.addEventListener("DOMContentLoaded", function () {
        // Get the canvas element
        var canvas = document.getElementById("renderCanvas");

        // Generate the Babylon.js engine
        var engine = new BABYLON.Engine(canvas, true);

        // Create the scene
        var createScene = function () {
          var scene = new BABYLON.Scene(engine);

          // Add a camera
          var camera = new BABYLON.ArcRotateCamera(
            "Camera",
            Math.PI / 2,
            Math.PI / 2,
            2,
            BABYLON.Vector3.Zero(),
            scene
          );
          camera.attachControl(canvas, true);
          /*
                    // cambia el color del fondo
                    var helper = scene.createDefaultEnvironment();
                    helper.setMainColor(BABYLON.Color3.Teal());
                    */
          // Add a light
          var light = new BABYLON.HemisphericLight(
            "light1",
            new BABYLON.Vector3(1, 1, 0),
            scene
          );
          light.intensity = 0.7;

          // Cargar los archivos si se proporcionan.
          if (modelFiles.length > 0) {
            var objFile = modelFiles.find((file) => file.uri.endsWith(".glb"));
            //var mtlFile = modelFiles.find(file => file.uri.endsWith('.mtl'));
            console.log("Model File URL:", objFile.url);
            if (objFile) {
              //&& mtlFile) {
              // Cargar el archivo .obj con el archivo .mtl.
              BABYLON.SceneLoader.ImportMesh(
                "",
                "",
                objFile.url,
                scene,
                function (meshes) {
                  // Ajustar la cámara para enfocar el modelo.
                  camera.target = meshes[0].position;
                },
                null,
                function (scene, message, exception) {
                  // Manejar errores de carga.
                  console.error("Error loading model:", message, exception);
                }
              );
            } else {
              // Si no hay archivos, mostrar una esfera por defecto.
              var sphere = BABYLON.MeshBuilder.CreateSphere(
                "sphere",
                { diameter: 1 },
                scene
              );
            }
          } else {
            // Si no hay archivos, mostrar una esfera por defecto.
            var sphere = BABYLON.MeshBuilder.CreateSphere(
              "sphere",
              { diameter: 1 },
              scene
            );
          }

          return scene;
        };

        var scene = createScene();

        // Render the scene
        engine.runRenderLoop(function () {
          scene.render();
        });

        // Resize the engine on window resize
        window.addEventListener("resize", function () {
          engine.resize();
        });
      });
    },
  };
})(Drupal, drupalSettings);
