(function (Drupal, drupalSettings) {
  "use strict";
  Drupal.behaviors.product3d = {
    attach: function (context, settings) {
      console.log("[viewer_3d]", "Load Module");

      if (drupalSettings.product3d === undefined) return;

      // Acceder a las variables del bloque.
      //var title = drupalSettings.product3d.title;
      //var description = drupalSettings.product3d.description;
      // Acceder a las URLs de los archivos subidos.
      var modelFiles = drupalSettings.product3d.model_files;

      // Mostrar las variables en la consola (opcional).
      //console.log("Title:", title);
      //console.log("Description:", description);
      console.log("Model File URL:", modelFiles);

      if (document.readyState !== "loading") {
        // DOM is already loaded
        canvasRender();
      } else {
        document.addEventListener("DOMContentLoaded", canvasRender());
      }

      function canvasRender() {
        // Get the canvas element
        var canvas = document.getElementById("renderCanvas");

        // Generate the Babylon.js engine
        var engine = new BABYLON.Engine(canvas, true);

        // Create the scene
        var createScene = function () {
          var scene = new BABYLON.Scene(engine);
          let modelMesh; // Variable para almacenar el modelo cargado.

          // Add a camera
          var camera = new BABYLON.ArcRotateCamera(
            "Camera",
            Math.PI / 2,
            Math.PI / 2,
            2,
            BABYLON.Vector3.Zero(),
            scene
          );
          camera.attachControl(canvas, true);

          // cambia el color del fondo
          /*
          var helper = scene.createDefaultEnvironment();
          helper.setMainColor(
            BABYLON.Color3.FromHexString(
              drupalSettings.product3d.background_color
            )
          );*/

          scene.clearColor = BABYLON.Color3.FromHexString(
            drupalSettings.product3d.background_color
          ); /*
          scene.backgroundColor = new BABYLON.Color3(0, 1, 0);
            */
          // Add a light
          var light = new BABYLON.HemisphericLight(
            "light1",
            new BABYLON.Vector3(1, 1, 0),
            scene
          );
          light.intensity = 0.7;

          // Cargar los archivos si se proporcionan.
          if (modelFiles.length > 0) {
            var objFile = modelFiles.find((file) => file.uri.endsWith(".glb"));
            //var mtlFile = modelFiles.find(file => file.uri.endsWith('.mtl'));
            console.log("Model File URL:", objFile.url);
            if (objFile) {
              //&& mtlFile) {
              // Cargar el archivo .obj con el archivo .mtl.
              BABYLON.SceneLoader.ImportMesh(
                "",
                "",
                objFile.url,
                scene,
                function (meshes) {
                  // Ajustar la cámara para enfocar el modelo.
                  camera.target = meshes[0].position;
                  modelMesh = meshes[1];
                  console.log("[viewer_3d] Meshes:", meshes);
                  /*
                  meshes.forEach(function (mesh) {
                    if (
                      mesh.material &&
                      mesh.material instanceof BABYLON.PBRMaterial
                    ) {
                      mesh.material.albedoColor = new BABYLON.Color3(1, 0, 0); // Rojo
                    }
                  });*/
                },
                null,
                function (scene, message, exception) {
                  // Manejar errores de carga.
                  console.error("Error loading model:", message, exception);
                }
              );

              // Aplicar el color seleccionado al material del modelo.
              const colorPicker = document.querySelector(
                '.color-picker input[type="color"]'
              );
              const applyButton = document.querySelector(
                ".color-picker button"
              );

              applyButton.addEventListener("click", function () {
                console.log("[viewer_3d]", "Click");
                const selectedColor = colorPicker.value;
                console.log(
                  "[viewer_3d]",
                  "Click Color",
                  modelMesh.material.name
                );
                if (
                  modelMesh.material &&
                  modelMesh.material instanceof BABYLON.PBRMaterial
                ) {
                  modelMesh.material.albedoColor =
                    BABYLON.Color3.FromHexString(selectedColor);
                  console.log("[viewer_3d]", "Cambio de color Albedo");
                } else if (modelMesh && modelMesh.material) {
                  modelMesh.material.diffuseColor =
                    BABYLON.Color3.FromHexString(selectedColor);
                  console.log("[viewer_3d]", "Cambio de color Diffuse");
                }
              });
            } else {
              // Si no hay archivos, mostrar una esfera por defecto.
              var sphere = BABYLON.MeshBuilder.CreateSphere(
                "sphere",
                { diameter: 1 },
                scene
              );
            }
          } else {
            // Si no hay archivos, mostrar una esfera por defecto.
            var sphere = BABYLON.MeshBuilder.CreateSphere(
              "sphere",
              { diameter: 1 },
              scene
            );
          }

          return scene;
        };

        var scene = createScene();

        // Render the scene
        engine.runRenderLoop(function () {
          scene.render();
        });

        // Resize the engine on window resize
        window.addEventListener("resize", function () {
          engine.resize();
        });
      }
    },
  };
})(Drupal, drupalSettings);
