<?php

namespace Drupal\viewer_3d\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\file\Entity\File;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'viewer3d_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "viewer3d_formatter",
 *   label = @Translation("Viewer 3D Formatter"),
 *   field_types = {
 *     "Viewer3D_field"
 *   }
 * )
 */
class Viewer3dFormatter extends FormatterBase
{
    /**
     * The file URL generator.
     *
     * @var \Drupal\Core\File\FileUrlGeneratorInterface
     */
    protected $fileUrlGenerator;

    /**
     * Constructs a CustomFormatter object.
     *
     * @param string $plugin_id
     *   The plugin_id for the formatter.
     * @param mixed $plugin_definition
     *   The plugin implementation definition.
     * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
     *   The definition of the field to which the formatter is associated.
     * @param array $settings
     *   The formatter settings.
     * @param string $label
     *   The formatter label display setting.
     * @param string $view_mode
     *   The view mode.
     * @param array $third_party_settings
     *   Any third party settings.
     * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
     *   The file URL generator.
     */
    public function __construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings, FileUrlGeneratorInterface $file_url_generator)
    {
        parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
        $this->fileUrlGenerator = $file_url_generator;
    }

    /**
     * {@inheritdoc}
     */
    public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
    {
        return new static(
            $plugin_id,
            $plugin_definition,
            $configuration['field_definition'],
            $configuration['settings'],
            $configuration['label'],
            $configuration['view_mode'],
            $configuration['third_party_settings'],
            $container->get('file_url_generator') // Inyectar el servicio
        );
    }

    /**
     * {@inheritdoc}
     */
    public function viewElements(FieldItemListInterface $items, $langcode)
    {
        $elements = [];

        foreach ($items as $delta => $item) {
            if (!empty($item->files)) {
                $file_ids = explode(',', $item->files);
                $file_urls = [];

                foreach ($file_ids as $fid) {
                    $file = File::load($fid);
                    if ($file) {
                        //$file_urls[] = $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri());//file_create_url($file->getFileUri());
                        $file_urls[] = [
                            'uri' => $file->getFileUri(),
                            'url' => $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri()),
                        ];
                    }
                }

                if (!empty($file_urls)) {

                    $elements[$delta] = [
                        '#theme' => 'viewer_3d',
                        // variables
                        '#title' => $config['title'] ?? 'Default Title', // Variable "title".
                        '#description' => $config['description'] ?? 'Default Description', // Variable "description".
                        '#background_color' => $config['background_color'] ?? '#FFFFFF',

                        '#file_urls' => $file_urls,
                        '#attached' => [
                            'library' => [
                                'viewer_3d/viewer_3d',
                            ],

                            // Pasar variables a js
                            'drupalSettings' => [
                                'product3d' => [
                                    'title' => $config['title'] ?? 'Default Title',
                                    'description' => $config['description'] ?? 'Default Description',
                                    'background_color' => $config['background_color'] ?? '#FFFFFF', // Nueva variable.
                                    'model_files' => $file_urls, // URL de los archivos 3D y materiales.
                                ],
                            ],
                        ],
                    ];
                }
            }
        }

        return $elements;
    }

}