<?php

namespace Drupal\viewer_3d\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'Viewer3D_widget' widget.
 *
 * @FieldWidget(
 *   id = "Viewer3D_widget",
 *   label = @Translation("3D Model Widget"),
 *   field_types = {"Viewer3D_field"}
 * )
 */
class Viewer3DWidget extends WidgetBase
{

    /**
     * {@inheritdoc}
     */
    public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state)
    {
        $element['files'] = [
            '#type' => 'managed_file',
            '#title' => $this->t('3D Model Files'),
            '#description' => $this->t('Upload the .glb file and associated textures.'),
            '#upload_location' => 'public://viewer_3d/',
            '#upload_validators' => [
                'FileExtension' => [
                    'extensions' => 'glb gltf mtl png jpg jpeg', // Extensiones permitidas.
                ],
            ],
            '#multiple' => TRUE, // Permitir subir múltiples archivos.
            '#default_value' => isset($items[$delta]->files) ? explode(',', $items[$delta]->files) : [],
        ];

        return $element;
    }

    /**
     * {@inheritdoc}
     */
    public function massageFormValues(array $values, array $form, FormStateInterface $form_state)
    {
        foreach ($values as &$value) {
            if (!empty($value['files'])) {
                $value['files'] = implode(',', $value['files']);
            }
        }
        return $values;
    }
}